<?php
/**
 * TestLink Open Source Project - http://testlink.sourceforge.net/
 * This script is distributed under the GNU General Public License 2 or later.
 *
 * @filesource $RCSfile: customCharts.php,v $
 * @version $Revision: 1.0 $
 * @author	Michal Grolmus
 *
 * This page show Progress Report.
 *
 * Revisions:
 *
 * ----------------------------------------------------------------------------------- */
require('../../config.inc.php');
require_once('common.php');
require_once('testplan.class.php');
require_once('displayMgr.php');
testlinkInitPage($db);

$templateCfg = templateConfiguration();
$args = init_args();
$results = array();
$gui = array();
$tproject_id = $_SESSION['testprojectID'];

// Get all test cases assigned to user without filtering by execution status
$opt = array('mode' => 'full_path');
$filters = initFilters($args);
$tplan_param = ($args->tplan_id) ? array($args->tplan_id) : testcase::ALL_TESTPLANS;

$tcase_mgr = new testcase($db);
$resultSet = $tcase_mgr->get_assigned_to_user($args->user_id, $tproject_id, $tplan_param, $opt, $filters);


$tplan_result = new testplan($db);
$can_manage_tprojects = has_rights($db,'mgt_modify_product');

if ($can_manage_tprojects)
{
	$user_id = null;
}
else
{
	$user_id = $_SESSION['userID'];
}

$tproject_id = $_SESSION['testprojectID'];
$tplan_id = $_REQUEST['tplan_id'];

$results = $tplan_result->get_progress_report_data($tproject_id, $tplan_id, $user_id,null,null,null,0,$args->latest);
$total_tc_count = $tplan_result->count_testcases($tplan_id);

$build = $tplan_result->getBuildByCriteria($tplan_id, 'maxReleaseDate');
$release_string = $build[0]["release_date"];
$release_date = strtotime($release_string);
$release_js = $tplan_result->getJsDateFormat($release_date);

$start_arr = min($results);
$start_string = date('Y-m-d', strtotime($start_arr[0]["ex_date"].'-1 day'));
$start_date = strtotime($start_string);

$end_arr = max($results);
$end_string = $end_arr[0]["ex_date"];
$end_date = strtotime($end_string);

$days_to_release = floor(($release_date - $start_date)/ (60 * 60 * 24));

$today_string = date('Y-m-d');
$today_date = strtotime($today_string);

if ($today_date > $release_date) {$end_for_range = $today_string;}
else {$end_for_range = $release_string;}

$dates = $tplan_result->date_range($start_string, $end_for_range, "+1 day", "Y-m-d");

$report_data = array (
  'p' => "",
  'b' => "",
  'f' => "",
  'plan' => ""
);

$last_status_count = array (
  'p' => 0,
  'b' => 0,
  'f' => 0
);

$days_to_release_counter = 0;

foreach ($dates as $day => $day_data) {
  if ((strtotime($day)) <= $today_date ) {
    foreach ($results[$day] as $day_status) {
      $last_status_count[$day_status["status"]] = $day_status["number"];
    }
    foreach ($last_status_count as $status => $count) {
      $report_data[$status] .= "[".$dates[$day]["jsDate"].", ".$count."], ";
    }
  }
  if ($days_to_release_counter <= $days_to_release) $report_data["plan"] .= "[".$dates[$day]["jsDate"].", ".
    (round((($total_tc_count/17)*$days_to_release_counter), 2))."], ";
  $days_to_release_counter++;
}


 
$columnsDefinition = report_status_headers();
$user = $_SESSION['currentUser']->getDisplayName();

$gui->tproject_id=$tproject_id;
$gui->tplan_id = $tplan_id;

// ----------------------------------------------------------------------------
$smarty = new TLSmarty;
$smarty->assign('columnsDefinition', $columnsDefinition);
$smarty->assign('buildResults',$results);
$smarty->assign('reportData',$report_data); 
$smarty->assign('releaseDate',$release_js);
$smarty->assign('user',$user);
$smarty->assign('gui',$gui);
$smarty->assign('latest',$args->latest);
$smarty->assign('printDate','');
$smarty->assign('tplan_id',$tplan_id);
$smarty->assign('tproject_id',$tproject_id);
$format = isset($_GET['format']) ? intval($_GET['format']) : null;
if (!isset($_GET['format']))
{
	tlog('$_GET["format"] is not defined');
	exit();
}
displayReport($templateCfg->template_dir . $templateCfg->default_template, $smarty, $format);

?>
<?php

/**Function report_status_headers to get the status verbose details from the status code like 'p' -> Pass, 'f' -> Failed etc
 * @return column -> status verbose map
 * @todo: We can include this functionality in existing code / class files
 * but dint knew where to do that
 */
function report_status_headers()
{

	$resultsCfg=config_get('results');
	$labels=$resultsCfg['status_label'];
	$labels_code=$resultsCfg['code_status'];

	foreach($labels_code as $key => $value)
	{
		if (!($key == 'a') and !($key == 'n'))
		{
			$code_status[$key]=$value;
		}
	}
	foreach($code_status as $key => $status_verbose)
	{
		$l18n_label = isset($labels[$status_verbose]) ? lang_get($labels[$status_verbose]) :
		lang_get($status_verbose);

		$colDefinition[$key]=$l18n_label;
	}
	return $colDefinition;
}
/*
 function:

 args :

 returns:

 */
function init_args()
{
    $args = new stdClass();

    $args->latest = isset($_REQUEST['latest']) ? $_REQUEST['latest'] : 1;
    return $args;
}

function initFilters($argsObj)
{
  $filters = array();
  
  $filters['tplan_status'] = $argsObj->show_inactive_tplans ? 'all' : 'active';
  $filters['build_status'] = $argsObj->show_closed_builds ? 'all' : 'open';
  
  if ($argsObj->build_id) 
  {
    $filters['build_id'] = $argsObj->build_id;
    
    // show assignments regardless of build and tplan status
    $filters['build_status'] = 'all';
    $filters['tplan_status'] = 'all';
  }
  return $filters;
}

?>
