<?php
// Set the JSON header
header("Content-type: text/json");

require('../../config.inc.php');
require_once('common.php');
require_once('requirements.inc.php');

$req_cfg = config_get('req_cfg');
list($req_spec_type_labels,$req_type_labels,$status_labels,$labels) = setUpLabels($req_cfg);
list($results_cfg,$status_code_map,$code_status_map,$eval_status_map) = setUpReqStatusCfg();

$colors = array("#ff0066", "#9900cc", "#0000ff", "#00cc66", "#6600cc", "#ff6699", "#3399ff", "#003366");



doDBConnect($db,database::ONERROREXIT);

class Series
{
    public $name;
    public $data = array();
    public $color;
    public $index;
}

$chart_number = $_REQUEST['ch_number'];
$tproject = $_REQUEST['tproject'];
$tplan = $_REQUEST['tplan'];
$element = $_REQUEST['element'];
$x_axis = $_REQUEST['x_axis'];
$group_by = $_REQUEST['group_by'];


$chart_options = array(
  "chart" => array(
    "renderTo" => "container".$chart_number,
    "type" => "column"
  ),
  "title" => array(
    "text" => "Test results by users"
  ),
  "xAxis" => array(
    "categories" => []//$results_to_json["categories"]
  ),
  "yAxis" => array(
    "min" => 1,
    "title" => array(
      "text" => "No. of TCs"
    ),
  ),
  "stackLabels" => array(
    "enabled" => true,
    "style" => array(
      "fontWeight" => "bold",
      "color" => "gray"
    )
  ),
  "tooltip" => array(
    "headerFormat" => '<b>{point.x}</b><br/>',
    "pointFormat" => '{series.name}: {point.y}<br/>Total: {point.stackTotal}'
  ),
  "plotOptions" => array(
    "column" => array(
      "stacking" => "normal",
      "dataLabels" => array(
        "enabled" => true,
        "color" => "white"
      )
    ),
  ),
  "series" => []//$results_to_json["series"]
);

switch ($element) {
  case "requirements":
    switch ($x_axis) {
      case "users":
        switch ($group_by) {
          case "req_stat":
              $results_to_json = requirementsByUser($tproject, $db);
              break;
        }
        break;
      case "type":
        switch ($group_by) {
          case "req_stat":
              $results_to_json = requirementsByType($tproject, $db);
              break;
        }
        break;
      case "builds":
        switch ($group_by) {
          case "req_stat":
              $results_to_json = testResultByBuild($tplan, $db);
              break;
        }
        break;
    }
  case "test_results":
    switch ($x_axis) {
      case "users":
        switch ($group_by) {
          case "exe_stat":
              $results_to_json = testResultByUser($tplan, $db);
              break;
        }
        break;
      case "priority":
        switch ($group_by) {
          case "exe_stat":
              $results_to_json = testResultByPriority($tplan, $db);
              break;
        }
        break;
      case "builds":
        switch ($group_by) {
          case "exe_stat":
              $results_to_json = testResultByBuild($tplan, $db);
              break;
        }
        break;
    }
}

$chart_options["xAxis"]["categories"] = $results_to_json["categories"];
$chart_options["series"] = $results_to_json["series"];
echo json_encode($chart_options);








function testResultByUser($tplan, $db){
  $sql = "SELECT
    users.login AS categories,
    ex.status AS name,
    count(*) AS data
  FROM ".DB_TABLE_PREFIX."executions as ex
  INNER JOIN (
    SELECT tcversion_id, MAX(execution_ts) AS latest_ex_date 
    FROM ".DB_TABLE_PREFIX."executions 
    GROUP BY tcversion_id
  ) AS latest_ex ON ex.tcversion_id = latest_ex.tcversion_id AND execution_ts = latest_ex_date
  JOIN ".DB_TABLE_PREFIX."users AS users ON ex.tester_id = users.id
  WHERE testplan_id = ".$tplan."
  GROUP BY login, status";
  
  $results = $db->get_recordset($sql);
  $results_to_json = sqlToJson($results);
   
  $results_to_json = editTestResultsSeries($results_to_json);

  return $results_to_json;  
}

function testResultByPriority($tplan, $db){
  $sql = "SELECT
    tcversions.importance AS categories,
    ex.status AS name,
    count(*) AS data
  FROM ".DB_TABLE_PREFIX."executions as ex
  INNER JOIN (
    SELECT tcversion_id, MAX(execution_ts) AS latest_ex_date 
    FROM ".DB_TABLE_PREFIX."executions 
    GROUP BY tcversion_id
  ) AS latest_ex ON ex.tcversion_id = latest_ex.tcversion_id AND execution_ts = latest_ex_date
  JOIN ".DB_TABLE_PREFIX."tcversions AS tcversions ON ex.tcversion_id = tcversions.id
  WHERE testplan_id = ".$tplan."
  GROUP BY importance, ex.status";
  
  $results = $db->get_recordset($sql);
  $results_to_json = sqlToJson($results);
  
  global $chart_options;
  $chart_options["title"]["text"] = "Test results by priority";
  
  foreach ($results_to_json["categories"] as &$category) {
    switch ($category) {
      case "1":
          $category = "Low";
          break;
      case "2":
          $category = "Medium";
          break;
      case "3":
          $category = "High";
          break;
    }
  }
  
   
  $results_to_json = editTestResultsSeries($results_to_json);
  
  return $results_to_json;  
}

function testResultByBuild($tplan, $db){
  $sql = "SELECT
    builds.name AS categories,
    ex.status AS name,
    count(*) AS data
  FROM ".DB_TABLE_PREFIX."executions as ex
  INNER JOIN (
    SELECT tcversion_id, MAX(execution_ts) AS latest_ex_date 
    FROM ".DB_TABLE_PREFIX."executions 
    GROUP BY tcversion_id
  ) AS latest_ex ON ex.tcversion_id = latest_ex.tcversion_id AND execution_ts = latest_ex_date
  JOIN ".DB_TABLE_PREFIX."builds AS builds ON builds.id = ex.build_id
  WHERE ex.testplan_id = ".$tplan."
  GROUP BY builds.id, ex.status";
  
  $results = $db->get_recordset($sql);
  $results_to_json = sqlToJson($results);
  
  global $chart_options;
  $chart_options["title"]["text"] = "Test results by builds";
   
  $results_to_json = editTestResultsSeries($results_to_json);

  return $results_to_json;  
}


function requirementsByUser($tproject, $db){
  $sql = "SELECT 
	users.login AS categories,
    reqv.status AS name,
    count(*) AS data 
    FROM ".DB_TABLE_PREFIX."nodes_hierarchy AS nhreq
    JOIN ".DB_TABLE_PREFIX."req_specs AS specs ON nhreq.parent_id = specs.id
    JOIN ".DB_TABLE_PREFIX."nodes_hierarchy AS nhreqv ON nhreqv.parent_id = nhreq.id
    JOIN ".DB_TABLE_PREFIX."req_versions AS reqv ON reqv.id = nhreqv.id
    JOIN ".DB_TABLE_PREFIX."users AS users ON users.id = reqv.author_id
WHERE specs.testproject_id = ".$tproject."
GROUP BY users.login, reqv.status";
  
  $results = $db->get_recordset($sql);
  $results_to_json = sqlToJson($results);
  
  global $chart_options;
  $chart_options["title"]["text"] = "Requirements by author";
  $chart_options["yAxis"]["title"]["text"] = "No. of RQs";
  
  $results_to_json = editRequirementsSeries($results_to_json);

  return $results_to_json;  
}

function requirementsByType($tproject, $db){
  $sql = "SELECT 
	  reqv.type AS categories,
    reqv.status AS name,
    count(*) AS data 
    FROM ".DB_TABLE_PREFIX."nodes_hierarchy AS nhreq
    JOIN ".DB_TABLE_PREFIX."req_specs AS specs ON nhreq.parent_id = specs.id
    JOIN ".DB_TABLE_PREFIX."nodes_hierarchy AS nhreqv ON nhreqv.parent_id = nhreq.id
    JOIN ".DB_TABLE_PREFIX."req_versions AS reqv ON reqv.id = nhreqv.id
    JOIN ".DB_TABLE_PREFIX."users AS users ON users.id = reqv.author_id
WHERE specs.testproject_id = ".$tproject."
GROUP BY reqv.type, reqv.status";
  
  $results = $db->get_recordset($sql);
  $results_to_json = sqlToJson($results);
  
  global $chart_options;
  $chart_options["title"]["text"] = "Requirements by type";
  $chart_options["yAxis"]["title"]["text"] = "No. of RQs";
    
  global $req_type_labels;
  foreach ($results_to_json["categories"] as &$category) {
    foreach ($req_type_labels as $number => $name) {    
      if ($category == $number) $category = $name;
    }
  }
   
  $results_to_json = editRequirementsSeries($results_to_json);

  return $results_to_json;  
}



function editTestResultsSeries ($results_to_json){  
  foreach ($results_to_json["series"] as $a_series) {
    switch ($a_series->name) {
      case "p":
          $a_series->name = "Passed";
          $a_series->color = "#00b050";
          $a_series->index = 1;
          break;
      case "b":
          $a_series->name = "Blocked";
          $a_series->color = "#ffc000";
          $a_series->index = 2;
          break;
      case "f":
          $a_series->name = "Failed";
          $a_series->color = "#ff0000";
          $a_series->index = 3;
          break;
    }
  }
  return $results_to_json;
}

function editRequirementsSeries ($results_to_json){
  global $status_labels;
  global $colors;
  $counter = 0; 
  foreach ($results_to_json["series"] as $a_series) {
    $a_series->color = $colors[$counter];
    $counter++;
    foreach ($status_labels as $char => $name) {
      if ($a_series->name == $char) {
        $a_series->name = $name;  
      }
    }
  }
  return $results_to_json;
}



function sqlToJson ($results) {

  $results_to_json = [];
  
  $results_to_json["categories"] = [];
  $results_to_json["series"] = [];
  $already_saved = false;
  
  foreach ($results as $row_num => $row) {
    foreach ($row as $column_name => $value) {
      if ($column_name == "categories") {
        if (!in_array($value, $results_to_json["categories"])) $results_to_json["categories"][] = $value; 
      }
      if ($column_name == "name") {
        $last_series_name = $value;
      }
      if ($column_name == "data") {
        foreach ($results_to_json["series"] as $a_series) {
          if ($a_series->name == $last_series_name) {
            array_push($a_series->data, intval($value));
            $already_saved = true;
            break;
          }
        }
        if (!$already_saved) {
          $one_series = new Series();
          $one_series->name = $last_series_name;
          array_push($one_series->data, intval($value));
          array_push($results_to_json["series"], $one_series);
        }
        $already_saved = false;
      }
    }
  }
  return $results_to_json;
}


function setUpLabels($reqCfg)
{
  $dummy = config_get('req_spec_cfg');
  $rsptlbl = init_labels($dummy->type_labels);

  $rtlbl = init_labels($reqCfg->type_labels);
  $slbl = init_labels($reqCfg->status_labels);

  $labels = init_labels( array('requirement' => null,'requirements' => null,
                               'type' => null,'req_availability' => null,
                               'linked_tcs' => null,'no_linked_tcs' => null,
                               'goto_testspec' => null,'design' => null,
                               'no_label_for_req_type'  => null, 'progress' => null,
                               'na' => 'not_aplicable', 'no_matching_reqs' => null,
                               'execution' => null,'no_srs_defined' => null,
                               'not_in_testplan' => null,'platform' => null,
                               'execution_history' => null, 'req_spec_short' => null));

  return array($rsptlbl,$rtlbl,$slbl,$labels);
}


function setUpReqStatusCfg()
{
  $results_cfg = config_get('results');
  
  $status_code_map = array();
  foreach ($results_cfg['status_label_for_exec_ui'] as $status => $label) 
  {
    $status_code_map[$status] = $results_cfg['status_code'][$status];
  }
  
  $code_status_map = array_flip($status_code_map);
  foreach ($code_status_map as $code => $status) 
  {
    $code_status_map[$code] = array('label' => lang_get($results_cfg['status_label'][$status]),
                                    'long_label' => lang_get("req_title_" . $status),
                                    'status' => $status,
                                    'css_class' => $status . '_text');
  }


  $eva = $code_status_map;
  
  // add additional states for requirement evaluation
  $evalbl = init_labels(array('partially_passed' => null, 'partially_passed_reqs' => null,
                              'uncovered' => null, 'uncovered_reqs' => null,
                              'passed_nfc' => null, 'passed_nfc_reqs' => null,
                              'failed_nfc' => null, 'failed_nfc_reqs' => null,
                              'blocked_nfc' => null, 'blocked_nfc_reqs' => null,
                              'not_run_nfc' => null, 'not_run_nfc_reqs' => null,
                              'passed' => null,
                              'partially_passed_nfc' => null, 'partially_passed_nfc_reqs' => null));

  $eva['partially_passed'] = array('label' => $evalbl['partially_passed'],
                                   'long_label' => $evalbl['partially_passed_reqs'],
                                   'css_class' => 'passed_text');

  $eva['uncovered'] = array('label' => $evalbl['uncovered'],
                            'long_label' => $evalbl['uncovered_reqs'],
                            'css_class' => 'not_run_text');

  $eva['p_nfc'] = array('label' => $evalbl['passed_nfc'],
                        'long_label' => $evalbl['passed_nfc_reqs'],
                        'css_class' => 'passed_text');
                        
  $eva['f_nfc'] = array('label' => $evalbl['failed_nfc'],
                        'long_label' => $evalbl['failed_nfc_reqs'],
                        'css_class' => 'failed_text');
                                    
  $eva['b_nfc'] = array('label' => $evalbl['blocked_nfc'],
                        'long_label' => $evalbl['blocked_nfc_reqs'],
                        'css_class' => 'blocked_text');
                        
  $eva['n_nfc'] = array('label' => $evalbl['not_run_nfc'],
                        'long_label' => $evalbl['not_run_nfc_reqs'],
                        'css_class' => 'not_run_text');
                        
  $eva['partially_passed_nfc'] = array('label' => $evalbl['partially_passed_nfc'],
                                       'long_label' => $evalbl['partially_passed_nfc_reqs'],
                                       'css_class' => 'passed_text');
  

  // add count for each status to show test progress
  foreach ($eva as $key => $status) 
  {
    $eva[$key]['count'] = 0;
  }

  return array($results_cfg,$status_code_map,$code_status_map,$eva);
}
?>